using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.IO;

//************** N A M E S P A C E ****************************************
namespace GameUtilities {

	//*********************************************************************
	//
	// FontGenerator Class
	//
	//*********************************************************************	
	public class FontGenarator {			
		/// <sumary>
		/// Convert a 32-bit number to a hex string with ls-byte first
		/// </sumary>
		private static string ToHexString(int num){
			string hex_chr = "0123456789ABCDEF";
			string str = "";
			str +=  hex_chr.Substring((num >> 4) & 0x0F,1) + hex_chr.Substring(num & 0x0F,1);
			return str;
		}
		
		/// <sumary>
		/// Build the static header for the output file
		/// </sumary>		
		private static string BuildHeaderText(int fontChaars) {
			string headerText = "'' Font generated by FontGenerator\n";
			headerText += "'' compatable with AIGeneric 1.2\n\n";
			headerText += "CON\n\n";
			headerText += "        AS_VIEWED               = true\n";
			headerText += "        FONT_CHARS              = "+fontChaars+"\n\n";
			
			headerText += "PUB GetPtrToFontTable\n\n";

			headerText += "        result                  := @fonttab\n\n";

			headerText += "pub define(c,c0,c1,c2,c3,c4,c5,c6,c7) | p \n";
			headerText += " p:=@fonttab+(c<<3)\n";
			headerText += " byte[p][0]:=c0\n";
			headerText += " byte[p][1]:=c1\n";
			headerText += " byte[p][2]:=c2\n";
			headerText += " byte[p][3]:=c3\n";
			headerText += " byte[p][4]:=c4\n";
			headerText += " byte[p][5]:=c5\n";
			headerText += " byte[p][6]:=c6\n";
			headerText += " byte[p][7]:=c7\n\n";
			headerText += "DAT\n\n";
			
			return headerText;
		}
		
		//*********** P U B L I C   F U N C T I O N S  ( M E T H O D S ) ******
	
		// Main (entrypoint)	
		public static void Main(string[] arguments) {
			if (arguments.Length > 0) {
				bool continueProcessing = true;
				
				string fileName = arguments[0];
				if (arguments.Length > 1) {
					string outputFileName = arguments[1];
					
					try {
						
						Bitmap bitmap = new Bitmap( fileName );
						if ((bitmap.Width != 256)) {
							Console.WriteLine("Error - source bitmap must be 256 pixels wide.");
							continueProcessing = false;
						}
						if ((bitmap.Height < 8)|| (bitmap.Height > 32)) {
							Console.WriteLine("Error - source bitmap must be at least 8 pixels high, and less than 32 pixels high.");
							continueProcessing = false;
						}	
						
						try {
							StreamWriter streamWriter = new StreamWriter(outputFileName);
						
							if (continueProcessing) {
								//the assumption is that each are 8x8 character blocks
								int totalCharacterRows = bitmap.Height / 8;
								int totalFontChars = totalCharacterRows * 32;
								
								streamWriter.Write(BuildHeaderText(totalFontChars));
								int currentCharacter = 0;
								for (int characterRow=0; characterRow < totalCharacterRows; characterRow++) {
									for (int characterLineWidth=0; characterLineWidth <32; characterLineWidth++) {

										for (int characterHeight = (characterRow * (int)8); characterHeight < (characterRow * (int)8) + (int)8; characterHeight++) {
											if ((currentCharacter == 0) && (characterHeight == (characterRow * (int)8))) {
												streamWriter.Write ("fonttab "); 
											} else {
												streamWriter.Write ("        "); 
											}
											streamWriter.Write ("byte byte ");
											string byteString = "";
											string picString = "";
											for (int characterWidth = (characterLineWidth * (int)8); characterWidth < (characterLineWidth * (int)8)+(int)8; characterWidth++) {
												
												Color pixelColor = bitmap.GetPixel(characterWidth, characterHeight);
												
												if ((pixelColor.R == 255) && (pixelColor.G == 255) && (pixelColor.B == 255)) {
													picString += "#";
													byteString += "1";
													
												} else {
													picString +=  ".";
													byteString += "0";
												}
												
											}
											//special case: put in AS_VIEWED if we are on 'space' (32)
											if ((currentCharacter == 32) && (characterHeight == (characterRow * (int)8))) {
												byteString = "AS_VIEWED";
											} else {
												byteString = "%" + byteString;
											}
											
											streamWriter.Write(byteString + "     ' "+ picString);
											if (characterHeight == (characterRow * (int)8)) streamWriter.Write (" $" + ToHexString(currentCharacter)); 
											streamWriter.WriteLine("");
										}
										streamWriter.WriteLine("");
										currentCharacter++;
									}
								}
								streamWriter.WriteLine("' End of data");
								streamWriter.Close();
								streamWriter = null;
							}
						
						} catch (Exception e){
							Console.WriteLine ("Error in creating output file " + outputFileName + " - " + e.ToString());
						}
						
						bitmap = null;
						
					} catch (Exception e) {
						Console.WriteLine("Error in processing image: " + e.ToString());
					}
				} else {
					Console.WriteLine ("Usage: FontGenerator.exe [bitmap_filename] [output_filename]");
				}
			} else { 
				Console.WriteLine ("Error - No output file specified.");
			}
		}
		
	}
}